@@ -1344,7 +1344,7 @@ fbCollector_t *fbCollectorAllocTLS(
 
 #endif /* HAVE_OPENSSL */
 
-#ifdef HAVE_SPREAD
+#if HAVE_SPREAD
 
 static gboolean fbCollectorSpreadOpen(
     fbCollector_t *collector,
@@ -1732,10 +1732,10 @@ void            fbCollectorFree(
     }
     collector->cotransClose(collector);
     fbCollectorClose(collector);
-    #ifdef HAVE_SPREAD
+#if HAVE_SPREAD
     if (collector->coclose == fbCollectorSpreadClose)
         fbConnSpreadFree( collector->stream.spread );
-    #endif
+#endif
     while (collector->udp_tail) {
         fbCollectorFreeUDPSpec(collector, collector->udp_tail);
     }
