@@ -96,15 +96,17 @@ struct fbExporter_st {
     /** Specifier used for stream open */
     union {
         fbConnSpec_t            *conn;
-        #ifdef HAVE_SPREAD
+#if HAVE_SPREAD
         fbSpreadSpec_t          *spread;
-        #endif
+#endif
         char                    *path;
     }                           spec;
     /** Current export stream */
     union {
         /** Buffered file pointer, for file transport */
         FILE                    *fp;
+        /** Buffer for data if providing own transport */
+        uint8_t                 *buffer;
         /**
          * Unbuffered socket, for SCTP, TCP, or UDP transport.
          * Also used as base socket for TLS and DTLS support.
@@ -122,6 +124,7 @@ struct fbExporter_st {
     SSL                         *ssl;
 #endif
     gboolean                    active;
+    size_t                      msg_len;
     fbExporterOpen_fn           exopen;
     fbExporterWrite_fn          exwrite;
     fbExporterClose_fn          exclose;
@@ -252,6 +255,71 @@ fbExporter_t    *fbExporterAllocFile(
     return exporter;
 }
 
+/**
+ * fbExporterOpenBuffer
+ *
+ *
+ */
+static gboolean fbExporterOpenBuffer(
+    fbExporter_t                *exporter,
+    GError                      **err)
+{
+    /* set active flag */
+    exporter->active = TRUE;
+
+    return TRUE;
+}
+
+/**
+ *fbExporterCloseBuffer
+ *
+ *
+ * @param exporter
+ *
+ */
+static void fbExporterCloseBuffer (
+    fbExporter_t              *exporter)
+{
+    exporter->active = FALSE;
+}
+
+static gboolean fbExporterWriteBuffer(
+    fbExporter_t             *exporter,
+    uint8_t                  *msgbase,
+    size_t                   msglen,
+    GError                   **err)
+{
+
+    memcpy(exporter->stream.buffer, msgbase, msglen);
+
+    exporter->msg_len = msglen;
+
+    return TRUE;
+
+}
+
+/**
+ * fbExporterAllocBuffer
+ *
+ *
+ *
+ */
+fbExporter_t *fbExporterAllocBuffer(
+    uint8_t                   *buf,
+    uint16_t                   bufsize)
+{
+    fbExporter_t *exporter = NULL;
+
+    exporter = g_slice_new0(fbExporter_t);
+    exporter->exwrite = fbExporterWriteBuffer;
+    exporter->exopen = fbExporterOpenBuffer;
+    exporter->exclose = fbExporterCloseBuffer;
+    exporter->mtu = bufsize;
+    exporter->stream.buffer = buf;
+
+    return exporter;
+}
+
 /**
  *fbExporterSetFilePath
  *
@@ -1427,3 +1495,16 @@ void fbExporterClose(
 {
     if (exporter->active && exporter->exclose) exporter->exclose(exporter);
 }
+
+/**
+ * fbExporterGetMsgLen
+ *
+ *
+ * @param exporter
+ */
+
+size_t fbExporterGetMsgLen(
+    fbExporter_t   *exporter)
+{
+    return exporter->msg_len;
+}
