@@ -691,7 +691,8 @@ static fbInfoElement_t defaults[] = {
     FB_IE_INIT_FULL("postIpPrecedence", 0, 257, 1,
                     FB_IE_F_ENDIAN | FB_IE_F_REVERSIBLE | FB_IE_IDENTIFIER,
                     0, 7, FB_UINT_8, NULL),
-    FB_IE_INIT_FULL("collectionTimeMilliseconds", 0, 258, 8, FB_IE_F_ENDIAN,
+    FB_IE_INIT_FULL("collectionTimeMilliseconds", 0, 258, 8,
+                    FB_IE_F_ENDIAN | FB_UNITS_MILLISECONDS,
                     0, 0, FB_DT_MILSEC, NULL),
     FB_IE_INIT_FULL("exportSctpStreamId", 0, 259, 2,
                     FB_IE_F_ENDIAN | FB_IE_F_ENDIAN, 0, 0, FB_UINT_16, NULL),
@@ -804,7 +805,7 @@ static fbInfoElement_t defaults[] = {
                     FB_IE_DEFAULT, 0, 0, FB_STRING, NULL),
     FB_IE_INIT_FULL("selectionSequenceId", 0, 301, 8,
                     FB_IE_F_ENDIAN | FB_IE_IDENTIFIER, 0, 0, FB_UINT_64, NULL),
-    FB_IE_INIT_FULL("selectorId", 0, 302, 2, FB_IE_F_ENDIAN | FB_IE_IDENTIFIER,
+    FB_IE_INIT_FULL("selectorId", 0, 302, 8, FB_IE_F_ENDIAN | FB_IE_IDENTIFIER,
                     0, 0, FB_UINT_64, NULL),
     FB_IE_INIT_FULL("informationElementId", 0, 303, 2,
                     FB_IE_F_ENDIAN | FB_IE_F_REVERSIBLE | FB_IE_IDENTIFIER, 0,
@@ -832,7 +833,7 @@ static fbInfoElement_t defaults[] = {
                     FB_UNITS_PACKETS, 0, 0, FB_UINT_32, NULL),
     FB_IE_INIT_FULL("samplingProbability", 0, 311, 8,
                     FB_IE_F_ENDIAN | FB_IE_F_REVERSIBLE | FB_IE_QUANTITY, 0, 0,
-                    FB_UINT_64, NULL),
+                    FB_FLOAT_64, NULL),
     FB_IE_INIT_FULL("dataLinkFrameSize", 0, 312, 2,
                     FB_IE_F_ENDIAN | FB_IE_F_REVERSIBLE, 0, 0,
                     FB_UINT_16, NULL),
@@ -1623,3 +1624,23 @@ const fbInfoElement_t *fbInfoModelIterNext(
     }
     return g_ptr_array_index(iter->model->ie_list, iter->index++);
 }
+
+const fbInfoElement_t     *fbInfoModelAddAlienElement(
+    fbInfoModel_t       *model,
+    fbInfoElement_t     *ex_ie)
+{
+    const fbInfoElement_t     *model_ie = NULL;
+
+    if (ex_ie == NULL) {
+        return NULL;
+    }
+    /* Information element not in model. Note it's alien and add it. */
+    ex_ie->ref.name = g_string_chunk_insert(model->ie_names,
+                                            "_alienInformationElement");
+    ex_ie->flags |= FB_IE_F_ALIEN;
+    fbInfoModelAddElement(model, ex_ie);
+    model_ie = fbInfoModelGetElement(model, ex_ie);
+    g_assert(model_ie);
+
+    return model_ie;
+}
