@@ -83,7 +83,8 @@ struct fbSession_st {
     fbNewTemplateCallback_fn    new_template_callback;
     /* Callback added to allow application to add context to a particular
        template */
-    fbTemplateCtxCallback_fn    tmpl_ctx_callback;
+    fbTemplateCtxCallback2_fn   tmpl_ctx_callback;
+    void                        *tmpl_app_ctx;
     /**
      * Mutex to guard the ext_ttab when using Spread.  Spread listens for new
      * group memberships.  On a new membership, the Spread transport will send
@@ -171,10 +172,10 @@ fbSession_t     *fbSessionAlloc(
     /* Allocate internal template table */
     session->int_ttab = g_hash_table_new(g_direct_hash, g_direct_equal);
 
-    #if HAVE_SPREAD
+#if HAVE_SPREAD
     /* this lock is needed only if Spread is enabled */
     pthread_mutex_init( &session->ext_ttab_wlock, 0 );
-    #endif
+#endif
 
     /* Reset session externals (will allocate domain template tables, etc.) */
     fbSessionResetExternal(session);
@@ -200,19 +201,90 @@ fbNewTemplateCallback_fn    fbSessionTemplateCallback(
     return session->new_template_callback;
 }
 
-void fbSessionAddTemplateCtxCallback(
-    fbSession_t                 *session,
-    fbTemplateCtxCallback_fn    callback)
+/**
+ * This function is used by fbSessionAddTemplateCtxWrapper().  It
+ * wraps an old-style fbTemplateCtxFree_fn into an
+ * fbTemplateCtxFree2_fn.  It works by having
+ * fbSessionTemplateCallbackWrapper() set the template's app_ctx to the
+ * old-style function, setting the free callback to this function, and
+ * this function call's the free function stashed in the app_ctx.
+ */
+static void fbTemplateCtxFreeWrapper(
+    void           *tmpl_ctx,
+    void           *app_ctx)
 {
-    session->tmpl_ctx_callback = callback;
+    fbTemplateCtxFree_fn free_fn = (fbTemplateCtxFree_fn)app_ctx;
+    free_fn(tmpl_ctx);
 }
 
-fbTemplateCtxCallback_fn fbSessionTemplateCtxCallback(
+/**
+ * This function is used by fbSessionAddTemplateCtxCallback().  It
+ * wraps an fbTemplateCtxCallback_fn into an
+ * fbTemplateCtxCallback2_fn.  It works by having
+ * fbSessionAddTemplateCtxCallback() stash the old-style callback
+ * function in the app_ctx pointer, setting the actual callback to
+ * this function.  This function then wraps and calls the old-style
+ * callback.
+ */
+static void fbSessionTemplateCallbackWrapper(
+    fbSession_t           *session,
+    uint16_t              tid,
+    fbTemplate_t          *tmpl,
+    void                  *app_ctx,
+    void                  **tmpl_ctx,
+    fbTemplateCtxFree2_fn *free_fn)
+{
+    /* Place to stash the old-style free function */
+    fbTemplateCtxFree_fn wrapped_free_fn = NULL;
+
+    /* Retrieve the old-style callback and call it. */
+    fbTemplateCtxCallback_fn wrapped_fn = (fbTemplateCtxCallback_fn)app_ctx;
+    wrapped_fn(session, tid, tmpl, tmpl_ctx, &wrapped_free_fn);
+
+    if (wrapped_free_fn == NULL) {
+        /* If the user provided no free function, propagate that */
+        *free_fn = NULL;
+    } else {
+        /* If the user provided an old-style free function, stash it
+         * in the template's app_ctx, and set the free_function to our
+         * wrapper free function. */
+        *free_fn = fbTemplateCtxFreeWrapper;
+        tmpl->app_ctx = (void *)wrapped_free_fn;
+    }
+}
+
+
+void fbSessionAddTemplateCtxCallback2(
+    fbSession_t                 *session,
+    fbTemplateCtxCallback2_fn    callback,
+    void                        *app_ctx)
+{
+    session->tmpl_ctx_callback = callback;
+    session->tmpl_app_ctx = app_ctx;
+}
+
+void fbSessionAddTemplateCtxCallback(
+    fbSession_t                 *session,
+    fbTemplateCtxCallback_fn     callback)
+{
+    /* Set the callback to the fbTemplateCtxCallback_fn wrapper
+     * callback, and set the old-style callback as its argument */
+    session->tmpl_ctx_callback = fbSessionTemplateCallbackWrapper;
+    session->tmpl_app_ctx = (void *)callback;
+}
+
+fbTemplateCtxCallback2_fn fbSessionTemplateCtxCallback(
     fbSession_t *session)
 {
     return session->tmpl_ctx_callback;
 }
 
+void *fbSessionTemplateCtxCallbackAppCtx(
+    fbSession_t *session)
+{
+    return session->tmpl_app_ctx;
+}
+
 void fbSessionAddTemplatePair(
     fbSession_t    *session,
     uint16_t        ext_tid,
@@ -316,13 +388,13 @@ void            fbSessionResetExternal(
                               NULL, (GDestroyNotify)g_hash_table_destroy);
 
     /* Null out stale external template table */
-    #if HAVE_SPREAD
+#if HAVE_SPREAD
     pthread_mutex_lock( &session->ext_ttab_wlock );
     session->ext_ttab = NULL;
     pthread_mutex_unlock( &session->ext_ttab_wlock );
-    #else
+#else
     session->ext_ttab = NULL;
-    #endif
+#endif
 
     /* Clear out the old sequence number table if we have one */
     if (session->dom_seqtab) {
@@ -397,9 +469,9 @@ void            fbSessionSetDomain(
     if (session->ext_ttab && (domain == session->domain)) return;
 
     /* Update external template table; create if necessary. */
-    #if HAVE_SPREAD
+#if HAVE_SPREAD
     pthread_mutex_lock( &session->ext_ttab_wlock );
-    #endif
+#endif
     session->ext_ttab = g_hash_table_lookup( session->dom_ttab,
                                              GUINT_TO_POINTER(domain) );
     if (!session->ext_ttab)
@@ -408,9 +480,9 @@ void            fbSessionSetDomain(
         g_hash_table_insert(session->dom_ttab, GUINT_TO_POINTER(domain),
                             session->ext_ttab);
     }
-    #if HAVE_SPREAD
+#if HAVE_SPREAD
     pthread_mutex_unlock( &session->ext_ttab_wlock );
-    #endif
+#endif
     /* Stash current sequence number */
     g_hash_table_insert(session->dom_seqtab,
                         GUINT_TO_POINTER(session->domain),
@@ -795,10 +867,10 @@ uint16_t        fbSessionAddTemplate(
     }
 
     /* Insert template into table */
-    #if HAVE_SPREAD
+#if HAVE_SPREAD
     if (!internal)
         pthread_mutex_lock( &session->ext_ttab_wlock );
-    #endif
+#endif
     g_hash_table_insert(ttab, GUINT_TO_POINTER((unsigned int)tid), tmpl);
     if (internal) {
         session->intTmplTableChanged = 1;
@@ -806,10 +878,10 @@ uint16_t        fbSessionAddTemplate(
         session->extTmplTableChanged = 1;
     }
 
-    #if HAVE_SPREAD
+#if HAVE_SPREAD
     if (!internal)
         pthread_mutex_unlock( &session->ext_ttab_wlock );
-    #endif
+#endif
     fbTemplateRetain(tmpl);
 
     return tid;
@@ -838,10 +910,10 @@ gboolean        fbSessionRemoveTemplate(
     }
 
     /* Remove template */
-    #if HAVE_SPREAD
+#if HAVE_SPREAD
     if (!internal)
         pthread_mutex_lock( &session->ext_ttab_wlock );
-    #endif
+#endif
     g_hash_table_remove(ttab, GUINT_TO_POINTER((unsigned int)tid));
 
     if (internal) {
@@ -854,10 +926,10 @@ gboolean        fbSessionRemoveTemplate(
 
     fBufRemoveTemplateTcplan(session->tdyn_buf, tmpl);
 
-    #if HAVE_SPREAD
+#if HAVE_SPREAD
     if (!internal)
         pthread_mutex_unlock( &session->ext_ttab_wlock );
-    #endif
+#endif
     fbTemplateRelease(tmpl);
 
     return ok;
@@ -875,15 +947,15 @@ fbTemplate_t    *fbSessionGetTemplate(
     /* Select a template table to get the template from */
     ttab = internal ? session->int_ttab : session->ext_ttab;
 
-    #if HAVE_SPREAD
+#if HAVE_SPREAD
     if (!internal)
         pthread_mutex_lock( &session->ext_ttab_wlock );
-    #endif
+#endif
     tmpl = g_hash_table_lookup(ttab, GUINT_TO_POINTER((unsigned int)tid));
-    #if HAVE_SPREAD
+#if HAVE_SPREAD
     if (!internal)
         pthread_mutex_unlock( &session->ext_ttab_wlock );
-    #endif
+#endif
 
     /* Check for missing template */
     if (!tmpl) {
@@ -946,9 +1018,9 @@ gboolean        fbSessionExportTemplates(
 {
     gboolean ret = TRUE;
 
-    #if HAVE_SPREAD
+#if HAVE_SPREAD
     pthread_mutex_lock( &session->ext_ttab_wlock );
-    #endif
+#endif
 
     g_clear_error(&(session->tdyn_err));
 
@@ -964,9 +1036,9 @@ gboolean        fbSessionExportTemplates(
         }
     }
 
-    #if HAVE_SPREAD
+#if HAVE_SPREAD
     pthread_mutex_unlock( &session->ext_ttab_wlock );
-    #endif
+#endif
 
     return ret;
 }
@@ -1002,6 +1074,7 @@ fbSession_t     *fbSessionClone(
        time it's too late and we've already missed some templates */
     session->new_template_callback = base->new_template_callback;
     session->tmpl_ctx_callback = base->tmpl_ctx_callback;
+    session->tmpl_app_ctx = base->tmpl_app_ctx;
 
     /* copy collector reference */
     session->collector = base->collector;
